#include "foofighter.h"

#ifdef SOUND
#include "/ave/iss/aom/aom_api.h"

struct sysob* aom = NULL;		// Audio manager class
#endif

/* Initialise the sampled sound player. */

typedef struct {
	char* filename;		// Name of file containing sample
	struct sysob* buffer;	// Sample buffer
} repositoryentry_t;

static repositoryentry_t repositorydata[] = {
	{DIRPREFIX "explosion.wav",	NULL}, // SOUND_EXPLOSION
	{DIRPREFIX "shot.wav",		NULL}, // SOUND_SHOT
	{DIRPREFIX "flash.wav",		NULL}, // SOUND_FLASH
};
#define REPOSITORY_LENGTH (sizeof(repositorydata)/sizeof(*repositorydata))
void init_audio(void)
{
#ifdef SOUND
	int i;

	/* Create the AOM session. */

	aom = SYSOB_CREATE_NM("ave/iss/aom/class");
	if (aom == NULL)
		fatalerror("failed to get AOM class");

	if (ave_iss_aom_session_created(aom) != ISS_STATUS_SUCCESS)
		fatalerror("failed to start AOM session");

	/* Load all the samples. */

	for (i=0; i<REPOSITORY_LENGTH; i++)
	{
		repositoryentry_t* o = &repositorydata[i];

		/* Create and load the audio buffer. */

		o->buffer = SYSOB_CREATE_NM("ave/iss/file/play/static/class");
		if (o->buffer == NULL)
			fatalerror("failed to create static audio buffer for <%s>",
					o->filename);

		if (ave_iss_file_play_static_open(o->buffer, aom,
		    o->filename, NULL) != ISS_STATUS_SUCCESS)
			fatalerror("failed to load sample <%s>", o->filename);
	}
#endif
}

/* Shutdown the sampled sound player. */

void shutdown_audio(void)
{
#ifdef SOUND
	int i;

	for (i=0; i<REPOSITORY_LENGTH; i++)
	{
		repositoryentry_t* o = &repositorydata[i];

		if (o->buffer)
			sys_ob_deref(o->buffer);
	}

	if (aom)
		sys_ob_deref(aom);

#endif
}

/* Play a sample. */

void play_sample(int sample)
{
#ifdef SOUND
	ave_iss_file_play_static_play(repositorydata[sample].buffer, NULL);
#endif
}

