#include "foofighter.h"

/* This code handles the user's bank and displaying it (and other messages) on
 * Messages go in the explosions layer, partly so we don't need another one
 * and partly so we get kewl effects of the explosions partially covering the
 * status messages. */

/* The width, height and colour of the progress indicator. */

#define PROGRESSIND_WIDTH 32 //radius
#define PROGRESSIND_HEIGHT 32 //radius
#define PROGRESSIND_COLOUR 0x8000FF00

/* The Y position of the text. */

#define TEXT_YCOORD 7

/* The Y position of the progress gadget. */

#define PROGRESSIND_YCOORD 10

/* The player's bank account and distance travelled. */

int money;
int distance;

/* The bank message display record. */

static struct ave_imgdlist_rec_string bankmsg;
static char bankmsgbuf[15];

/* The progress message display records. The numeric part and the graphic part
 * are seperate records for simplicity. */

static struct ave_imgdlist_rec_string progressmsg;
static char progressmsgbuf[8];
static struct ave_imgdlist_rec_user progressind;

/* The string display routines need a ruler to tell it where the tab stops
 * are. We don't use this, so the ruler is empty.
 */

static int ruler[] = {0};

/* Utility function that sets the string a text dlistrec displays. Because the
 * dlistrec needs to know the width and height of the thing inside, it must be
 * calculated when we set it. There isn't any overhead because the size
 * calculation will cache values used later when the string is actually drawn.
 */

static void settext(struct ave_imgdlist_rec_string* dlistrec, char* msg,
		int rightaligned)
{
	/* First, copy the string data. */

	if (dlistrec->pString != msg)
		strncat(dlistrec->pString, msg, dlistrec->charcnt);

	/* Now calculate the size. */

	{
		ave_font_dimensions_t r = ave_font_strsize(font, msg, NULL);
		dlistrec->rec.width = r.width;
		dlistrec->rec.height = r.height;
	}

	/* Right aligned? */

	if (rightaligned > 0)
		dlistrec->rec.x = rightaligned - dlistrec->rec.width;
}

/* The progress indicator is a user object because there isn't a dlist type
 * for arcs. This routine does the actual redraw. */

static void progressind_redraw(ave_avo_t* avo, ave_ctx_t* ctx,
		ave_imgdlist_rec_t* dlistrec)
{
	/* Draw the arc. This routine isn't as complicated as it looks. */

	ave_ctx_afarc(ctx,
		dlistrec->x, dlistrec->y,
		PROGRESSIND_COLOUR,
		PROGRESSIND_WIDTH, PROGRESSIND_HEIGHT,
		90, ((distance % 100) * 360) / 100);
}

/* Initialise the bank. */

void init_bank(void)
{
	/* Reset the bank counters first. */

	reset_bank();

	/* Set up the dlist record for the money message. Yes, this is
	 * ghastly, but trust me. */

	bankmsg.rec.type = IMGDLIST_TYPE_STRING;
	bankmsg.rec.x = 5;
	bankmsg.rec.y = TEXT_YCOORD;
	bankmsg.flags = TRF_INK;
	bankmsg.ink = 0x00FFFFFF;
	bankmsg.paper = 0xFF000000;
	bankmsg.pixrule = PIX_RULE_ALPHA;
	bankmsg.pString = bankmsgbuf;
	bankmsg.pFont = font;
	bankmsg.charcnt = sizeof(bankmsgbuf);
	bankmsg.pRuler = ruler;
	elate_reftool("ave/util/str/r_utf8", bankmsg.pDecoder);

	adjust_bank(0);

	/* Next, the progress text. */

	progressmsg.rec.type = IMGDLIST_TYPE_STRING;
	progressmsg.rec.x = 0; // filled in later
	progressmsg.rec.y = TEXT_YCOORD;
	progressmsg.flags = TRF_INK;
	progressmsg.ink = 0x00FFFFFF;
	progressmsg.paper = 0xFF000000;
	progressmsg.pixrule = PIX_RULE_ALPHA;
	progressmsg.pString = progressmsgbuf;
	progressmsg.pFont = font;
	progressmsg.charcnt = sizeof(progressmsgbuf);
	progressmsg.pRuler = ruler;
	elate_reftool("ave/util/str/r_utf8", progressmsg.pDecoder);

	/* And the graphical progress indicator. */

	progressind.rec.type = IMGDLIST_TYPE_USER;
	progressind.rec.x = WIN_WIDTH - 5 - PROGRESSIND_WIDTH;
	progressind.rec.y = PROGRESSIND_YCOORD;
	progressind.rec.width = PROGRESSIND_WIDTH;
	progressind.rec.height = PROGRESSIND_HEIGHT;
	progressind.callback = progressind_redraw;

	adjust_progress(0);

	/* And add the records to the explosions display list. */

	ave_imgdlist_addrechead(explosiondisplaylist,
			(ave_imgdlist_rec_t*) &bankmsg);

	ave_imgdlist_addrechead(explosiondisplaylist,
			(ave_imgdlist_rec_t*) &progressmsg);

	ave_imgdlist_addrechead(explosiondisplaylist,
			(ave_imgdlist_rec_t*) &progressind);
}

void reset_bank(void)
{
	/* The player starts with 1000 euros in the bank and has travelled 0
	 * kilometres. */
	
	money = 1000;
	distance = 0;
}
	
/* Pay/fine the player. */

void adjust_bank(int amount)
{
	money += amount;
	if (money < 0)
		money = 0;
	
	// The dodgy character in the next line is supposed to be a UTF8 Euro
	// symbol. Unfortunately none of the fonts *have* one, so I've
	// replaced with a EUR string. Blast.

	sprintf(bankmsgbuf, "EUR %d", money);
	settext(&bankmsg, bankmsgbuf, 0);
}

/* Move the player on a bit. The delta is in decametres, by the way. */

void adjust_progress(int delta)
{
	distance += delta;

	sprintf(progressmsgbuf, "%dkm", distance/100);
	settext(&progressmsg, progressmsgbuf, WIN_WIDTH - PROGRESSIND_WIDTH - 15);
}

