#include "foofighter.h"

/* Drifters drift across the screen at a constant velocity and are destroyed
 * when they leave the screen. They're used for a variety of purposes. */

/* Process one frame of scenery. */

void tick_drifter(object_t* object)
{
	/* Update the sprite's frame. */

	object->frame++;

	/* But wait! Has the ship moved off the screen? */

	if (((object->xv < 0) && (object->rx < -100<<16)) ||
	    ((object->xv > 0) && (object->rx > (WIN_WIDTH+100)<<16)) ||
	    ((object->yv < 0) && (object->ry < -100<<16)) ||
	    ((object->yv > 0) && (object->ry > (WIN_HEIGHT+100)<<16)))
	{
		/* Only penalise the player if it goes off the bottom. */
		if (object->ry > (WIN_HEIGHT+100)<<16)
			adjust_bank(-object->penalty);
		destroy_object(object);
		return;
	}

	/* Update the sprite. */

	update_sprite(object);
}

object_t* create_drifter(int sprite, ave_img_t* layer, int x, int y,
		int xv, int yv)
{
	object_t* object = alloc_object();
	if (!object)
		return NULL;

	object->sprite = sprite;
	object->frame = 0;
	object->rx = x;
	object->ry = y;
	object->xv = xv;
	object->yv = yv;
	object->layer = layer;

	object->damageable = 1;
	object->deadly = 1;
	object->tick = tick_drifter;
	object->deinit = NULL;

	init_sprite(object);
	update_sprite(object);
	return object;
}

