#include "foofighter.h"

/* Handles the overall game mechanics. */

/* Create a new player object (the old one must be destroyed or Weird Things
 * will happen). */

void new_player(void)
{
	int i;
	int olddistance;

	/* Create the player. Initially the player is off the bottom of the
	 * screen, with a negative y velocity so that it pops on when the game
	 * is started, making a generally kewl effect. */
	
	player = create_player((WIN_WIDTH/2)<<16, (WIN_HEIGHT+WIN_HEIGHT/2)<<16);
	player->yv = -100<<16;

	/* We give the player one second of invulnerability. We make the
	 * progress indicator flash madly by fiddling with the distance.
	 * Cheating, but it works. */

	olddistance = distance;
	distance = distance - (distance % 100);
	for (i=0; i<FRAMERATE; i++)
	{
		if (i & 1)
			adjust_progress(-99);
		else
			adjust_progress(99);

		animator_tick();
		ave_avo_update((ave_avo_t*) content);
		aveevent(kn_time_get() + 1E9/FRAMERATE);
	}
	distance = olddistance;
}

/* The player died. Run through the dead player animation and spawn a new one,
 * if it has the cash. */

void player_died(void)
{
	int i;

	/* Pause for two seconds. */

	for (i=0; i<FRAMERATE; i++)
	{
		animator_tick();
		ave_avo_update((ave_avo_t*) content);
		aveevent(kn_time_get() + 1E9/FRAMERATE);
	}

	/* Each ship costs 500 euros. */

	if (money > 500)
	{
		set_statusmsg("New vehicle being issued");
		adjust_bank(-500);
		new_player();
		return;
	}

	/* The player can't afford another ship. Tell the player, pause again,
	 * and quit the game. */

	set_statusmsg("You have gone bankrupt");

	for (i=0; i<FRAMERATE*2; i++)
	{
		animator_tick();
		ave_avo_update((ave_avo_t*) content);
		aveevent(kn_time_get() + 1E9/FRAMERATE);
	}

	shutdown();
	exit(0);
}

/* Create a new thing to shoot at. */

void new_object(void)
{
	/* We get a new type for every 10 km. */

	int type = randomrange(min((distance / 1000)+1, 6));

	/* The drift velocity varies over the first 40 km up to a maximum of 4.
	 * */

	int dvel = randomrange(min((distance / 1000)+1, 4))+1;

	/* Now create the object. */
	
	static int sprite[] = {
		SPRITE_SPINNER,
		SPRITE_CUBE,
		SPRITE_RINGBALL,
		SPRITE_D12,
		SPRITE_SPIKEDWHEEL,
		SPRITE_STONEBALL
	};
	object_t* obj = create_drifter(sprite[type],
			shipdisplaylist,
			interpolate(32, WIN_WIDTH-32, 0, 1000, randomrange(1000))<<16, -100<<16,
			0<<16, dvel<<16);

	/* We set the type here. */
	switch(type)
	{
		case 0: /* Basic spinner */
			obj->damageable = 1;
			obj->bonus = 10;
			obj->penalty = 50;
			break;

		case 1: /* Colour cube */
			obj->damageable = 2;
			obj->bonus = 20;
			obj->penalty = 70;
			break;

		case 2: /* Halo ball */
			obj->damageable = 3;
			obj->bonus = 30;
			obj->penalty = 100;
			break;

		case 3: /* Revenge of the RPGers */
			obj->damageable = 4;
			obj->bonus = 50;
			obj->penalty = 150;
			break;

		case 4: /* James Bond has nothing on this */
			obj->damageable = 6;
			obj->bonus = 100;
			obj->penalty = 200;
			break;

		case 5: /* Captain Scarlet! */
			obj->damageable = 20;
			obj->bonus = 1000;
			obj->penalty = 1000;
			break;
	}
}

/* Start a new game. */

void start_game(void)
{
	new_player();
	reset_bank();

	set_statusmsg("Entering debris field");

	/* The main game loop. */

	for (;;)
	{
		/* Is the player alive and has money? */

		if ((player == NULL) || (money <= 0))
			player_died();

		/* Shunt the player on a bit. */

		adjust_progress(1);

		/* Create a Random Thing to Shoot At(TM). The inverse-rate-value-thingy
		 * drops linearly from 60 to 30 over the first 150km of distance. */

		{
			int rate = interpolate(60, 30, 0, 10000, distance);
			if (randomrange(rate) == 0)
				new_object();
		}

		/* Perform any collision detections. This may create/destroy objects. */

		collision_detect();

		/* Do any other automatic animation. */

		animator_tick();

		/* Everything's changed, but nothing's updated. We need to force a
		 * redraw. Normally this is done by calling update on all the things
		 * that have changed. However, because all our display lists are
		 * stacked on top of each other, we can optimise by only calling
		 * update on the thing at the bottom. Everything on top of it will be
		 * automatically refreshed. */

		ave_avo_update((ave_avo_t*) content);

		/* And wait for the next frame to come along. */

		aveevent(kn_time_get() + 1E9/FRAMERATE);
	}
}


