#include "foofighter.h"

/* This file contains the generic code to handle all the objects in the game. */

/* We store the list of active objects in an ELATE_LIST. This allows us to
 * iterate through it and add and remove entries very quickly. Because we need
 * to allocate a new entry every time we add something that's a little
 * expensive, but we can get round it by using a custom allocator. No need in
 * this case. */

ELATE_LIST_HEAD objectlisthead;

/* We cache the player's object for easy access. */

object_t* player;

/* Initialise. */

void init_objects(void)
{
	/* Initialise the object list. */

	elate_initlist(&objectlisthead);
}

/* Process one frame worth of all the ships. */

void animate_objects(void)
{
	ELATE_LIST_NODE* n = elate_headend(&objectlisthead);
	ELATE_LIST_NODE* nextn = elate_succ(n);
	while (elate_succnode(nextn, n))
	{
		object_t* o = (object_t*) n;
		o->tick(o);
	}
}

/* Allocate an object and add it to the list. Warning: the object will be
 * partially initialised! You must finish initialisation before starting the
 * next frame! Or HORRIBLE things will happen!
 *
 * All fields in the object are initialised to zero.
 */

object_t* alloc_object(void)
{
	object_t* o = (object_t*) calloc(1, sizeof(object_t));
	elate_addhead(&objectlisthead, (ELATE_LIST_NODE*) o);
	return o;
}

/* Destroy an object. */

void destroy_object(object_t* object)
{
	/* Unhook the object from any layer it's on. */

	if (object->layer)
		ave_imgdlist_subrec(object->layer,
				(ave_imgdlist_rec_t*) &object->drec);

	/* Remove it from the object list. */

	elate_remove((ELATE_LIST_NODE*) object);

	/* And free it. */

	free(object);
}

