#include "foofighter.h"

/* Contains all the player-related code. */

/* Process one frame of the player. */

void tick_player(object_t* object)
{
	/* Adjust the ship's velocity according to the player's input. */
	
	if (key_left)
		object->xv -= ACCELERATION;
	if (key_right)
		object->xv += ACCELERATION;
	if (key_up)
		object->yv -= ACCELERATION;
	if (key_down)
		object->yv += ACCELERATION;

	/* If the player's run out of cash, accelerate him off the bottom of
	 * the screen. */

	if (money == 0)
		object->yv += 2*ACCELERATION;

	/* Damp the player's velocity. */

	object->xv -= (object->xv*DAMPING) / 100;
	object->yv -= (object->yv*DAMPING) / 100;

	/* Adjust the ship sprite according to the amount of turn. */

	{
		int i = (object->xv >> 16) / 2;
		i += 5;
		if (i < 0)
			i = 0;
		if (i > 10)
			i = 10;

		object->frame = i;
	}

	/* If the ship's moving off the screen, stop it; but don't change the
	 * velocity, because that changes the ship sprite abruptly and looks
	 * bad. Just halt it in its tracks.
	 *
	 * This clipping is done *before* the sprite is moved in
	 * update_sprite() for a reason. If the ship is right at the left edge
	 * of the clipping bounds, but trying to move left, then the X ordinate
	 * is set to 0 here and then decremented due to the ship's velocity
	 * later. The next frame, the X ordinate is set to zero again, moving
	 * it back. This means that the ship can go a few pixels beyond the
	 * boundary if it's pushing hard. In practice this lends a rather nice
	 * springy effect to the screen bounds, which I discovered completely
	 * by accident.
	 *
	 * Nasty hack: we only do the bounds check if the player has money.
	 * This is so the player can get properly sucked off the screen when
	 * its ship gets reposessed.
	 */

	if (money > 0)
	{
		/* The order of this is important; I'm reusing old values to
		 * try and make it efficient. */

		int i = (object->drec.rec.width/2)<<16;
		if ((object->rx < i)
		    && (object->xv < 0))
			object->rx = i;

		i = (WIN_WIDTH<<16) - i;
		if ((object->rx > i)
		    && (object->xv > 0))
			object->rx = i;

		i = (object->drec.rec.height/2)<<16;
		if ((object->ry < i)
		    && (object->yv < 0))
			object->ry = i;

		i = (WIN_HEIGHT<<16) - i;
		if ((object->ry > i)
		    && (object->yv > 0))
			object->ry = i;
	}
		
	/* Check whether we're firing or not. */

	if (object->u.player.firing)
	{
		if (!key_fire)
		{
			/* Firing, but user has just let go of key. Reset. */

			object->u.player.firing = 0;
		}
	}
	else
	{
		if (key_fire)
		{
			/* Not firing, user has pressed key. Generate bullet
			 * and lock mechanism. Bullets appear at the top of
			 * the player icon. */

			int ry = object->ry - (object->drec.rec.height/2 << 16);
			play_sample(SOUND_SHOT);
			create_bullet(0, object->rx, ry, 0, -BULLETSPEED);
			object->u.player.firing = 1;
		}
	}

	/* And update. */

	update_sprite(object);
}

/* Create a new player object. */

object_t* create_player(int x, int y)
{
	object_t* object = alloc_object();
	if (!object)
		return NULL;

	object->sprite = SPRITE_SHIP;
	object->frame = 5;
	object->rx = x;
	object->ry = y;
	object->xv = 0;
	object->yv = 0;
	object->layer = shipdisplaylist;

	object->tick = tick_player;
	object->deinit = NULL;

	init_sprite(object);
	update_sprite(object);
	return object;
}


