#include "foofighter.h"

/* Bail out with an error. */

void fatalerror(char* msg, ...)
{
	va_list ap;
	
	va_start(ap, msg);
	fprintf(stderr, "Fatal error: ");
	vfprintf(stderr, msg, ap);
	fprintf(stderr, "\n");
	va_end(ap);

	shutdown();
	exit(1);
}

/* Return a random number in the range 0..(size-1). rand() is a dreadful
 * random number generator; _Numerical Recipes in C_, one of the definitive
 * works on C algorithms, describes it with the phrase 'totally botched'. So
 * we're going to use our own. This is adapted from page 279 of that book and
 * is the Park & Miller random number generator. */

#define IA 16807
#define IM 2147483647
#define IQ 127773
#define IR 2836
static int seed = 0;
int randomrange(int size)
{
	long k, ans;

	/* Self-seed off the clock. */

	if (seed == 0)
		seed = (int) (kn_time_get() & 0xFFFFFFFF);

	k = seed / IQ;
	seed = IA * (seed - k*IQ) - (IR * k);
	if (seed < 0)
		seed += IM;
	ans = seed % size;
	return ans;
}

/* Linearly interpolate A to B according to timebase a to b, with value z. */

int interpolate(int A, int B, int a, int b, int z)
{
	double g;
	if (z < a)
		return A;
	if (z > b)
		return B;
	g = ((double)B - (double)A) / ((double)b - (double)a);
	return A + g*((double)(z-a));
}

/* Linearly interpolate RGB A to B according to, etc. */

int interpolateRGB(int A, int B, int a, int b, int z)
{
	int Ar = (A >> 16) & 0xFF;
	int Ag = (A >> 8) & 0xFF;
	int Ab = (A >> 0) & 0xFF;
	int Br = (B >> 16) & 0xFF;
	int Bg = (B >> 8) & 0xFF;
	int Bb = (B >> 0) & 0xFF;
	int Zr = interpolate(Ar, Br, a, b, z);
	int Zg = interpolate(Ag, Bg, a, b, z);
	int Zb = interpolate(Ab, Bb, a, b, z);
	return (Zr<<16) | (Zg<<8) | (Zb<<0);
}

